package com.io.yutian.elementoriginlib.nbt;

import com.io.yutian.elementoriginlib.serialize.serializers.ItemStackDeserializer;
import net.minecraft.core.component.DataComponents;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.world.item.component.CustomData;
import org.bukkit.craftbukkit.inventory.CraftItemStack;
import org.bukkit.inventory.ItemStack;
import org.jetbrains.annotations.NotNull;

import javax.annotation.Nullable;
import java.util.function.Consumer;

public class NBTItem {

    private ItemStack originalItemStack;
    private net.minecraft.world.item.ItemStack nmsItemStack;

    protected ItemStack resultItemStack;

    public NBTItem(ItemStack itemStack) {
        this.originalItemStack = itemStack;
        this.nmsItemStack = CraftItemStack.asNMSCopy(itemStack);
    }

    @NotNull
    public TagCompound getTag() {
        CompoundTag tag = nmsItemStack.getComponents().get(DataComponents.CUSTOM_DATA).getUnsafe();
        if (tag == null) {
            tag = new CompoundTag();
        }
        return (TagCompound) ITag.as(tag);
    }

    public boolean has(String key) {
        TagCompound tagCompound = getTag();
        return tagCompound.hasKey(key);
    }

    public boolean has(String key, int type) {
        TagCompound tagCompound = getTag();
        return tagCompound.hasKey(key, type);
    }

    @Nullable
    public ITag get(String key) {
        TagCompound tagCompound = getTag();
        return tagCompound.hasKey(key) ? tagCompound.get(key) : null;
    }

    public void editTag(Consumer<TagCompound> consumer) {
        TagCompound tag = getTag();
        consumer.accept(tag);
        setTag(tag);
    }

    public void setTag(TagCompound tagCompound) {
        nmsItemStack.set(DataComponents.CUSTOM_DATA, CustomData.of((CompoundTag) ITag.asNMS(tagCompound)));
        resultItemStack = CraftItemStack.asBukkitCopy(nmsItemStack);
    }

    public ItemStack getOriginalItemStack() {
        return originalItemStack;
    }

    public ItemStack getItemStack() {
        return resultItemStack;
    }

    public net.minecraft.world.item.ItemStack getNMSItemStack() {
        return nmsItemStack;
    }

    public static NBTItem build(ItemStack itemStack) {
        return new NBTItem(itemStack);
    }

    public static NBTItem build(TagCompound tagCompound) {
        net.minecraft.world.item.ItemStack nmsItemStack = net.minecraft.world.item.ItemStack.parse(ItemStackDeserializer.getRegistry(), ITag.asNMS(tagCompound)).get();
        return new NBTItem(CraftItemStack.asBukkitCopy(nmsItemStack));
    }

}
